@@ -54,7 +54,7 @@ from common.hstr import \
     minimum as _min,\
     clean as _clean
 
-# Global definitions
+# Enum definitions
 
 
 class Scope(Enum):
@@ -91,6 +91,14 @@ class State(Enum):
     Negotiable = 0x0020
     Specific = 0x0040
 
+# Exception definition
+
+
+class PcscError(Exception):
+    pass
+
+
+# Functions definitions
 
 def establish_context(scope: Scope):
     """establish_context():
@@ -100,10 +108,10 @@ def establish_context(scope: Scope):
     hresult, hcontext = _SCardEstablishContext(dw_scope)
     if hresult != _SCARD_S_SUCCESS:
         err = F'Failed to establish context ({_SCardGetErrorMessage(hresult)})'
-        logging.info(err)
-        raise Exception(err)
+        logging.debug(err)
+        raise PcscError(err)
 
-    logging.info(F"Context established with scope={scope}")
+    logging.debug(F"Context established with scope={scope}")
     return hcontext
 
 
@@ -116,10 +124,10 @@ def list_readers(hcontext, readergroups=None):
     hresult, readers = _SCardListReaders(hcontext, readergroups)
     if hresult != _SCARD_S_SUCCESS:
         err = F'Failed to list readers ({_SCardGetErrorMessage(hresult)})'
-        logging.info(err)
-        raise Exception(err)
+        logging.debug(err)
+        raise PcscError(err)
 
-    logging.info(F"Readers listed: {', '.join(readers)}")
+    logging.debug(F"Readers listed: {', '.join(readers)}")
     return readers
 
 
@@ -133,11 +141,11 @@ def connect(hcontext, reader, share_mode: ShareMode, preferred_protocols: Protoc
         hcontext, reader, dw_share_mode, dw_preferred_protocols)
     if hresult != _SCARD_S_SUCCESS:
         err = F'Unable to connect ({_SCardGetErrorMessage(hresult)})'
-        logging.info(err)
-        raise Exception(err)
+        logging.debug(err)
+        raise PcscError(err)
 
     protocol = Protocol(dw_active_protocol)
-    logging.info(F"Connected with mode={share_mode}, protocol={protocol}")
+    logging.debug(F"Connected with mode={share_mode}, protocol={protocol}")
     return hcard, protocol
 
 
@@ -153,11 +161,11 @@ def reconnect(hcard, share_mode: ShareMode, preferred_protocols: Protocol, initi
 
     if hresult != _SCARD_S_SUCCESS:
         err = F'Unable to reconnect ({_SCardGetErrorMessage(hresult)})'
-        logging.info(err)
-        raise Exception(err)
+        logging.debug(err)
+        raise PcscError(err)
 
     active_protocol = Protocol(dw_active_protocol)
-    logging.info(
+    logging.debug(
         F"Reconnected with mode={share_mode}, disposition={initialization}, protocol={active_protocol}")
     return active_protocol
 
@@ -169,15 +177,15 @@ def status(hcard):
 
     if hresult != _SCARD_S_SUCCESS:
         err = F'Unable to get current reader status ({_SCardGetErrorMessage(hresult)})'
-        logging.info(err)
-        raise Exception(err)
+        logging.debug(err)
+        raise PcscError(err)
 
     protocol = Protocol(dw_protocol)
     state = []
     for scard_state in State:
         if dw_state & scard_state.value:
             state.append(scard_state)
-    logging.info(
+    logging.debug(
         F"Card status: reader='{reader}', state={state}, protocol={protocol}, ATR={_to_hstr(atr)}")
     return reader, state, protocol, _to_hstr(atr)
 
@@ -193,15 +201,15 @@ def transmit(hcard, protocol: Protocol, apdu: str):
     hresult, response_bytes = _SCardTransmit(hcard, pio_send_pci, apdu_bytes)
     if hresult != _SCARD_S_SUCCESS:
         err = F'Failed to transmit ({_SCardGetErrorMessage(hresult)})'
-        logging.info(err)
-        raise Exception(err)
+        logging.debug(err)
+        raise PcscError(err)
 
     response = _to_hstr(response_bytes)
     data = response[0:-4]
     SW1 = response[-4:-2]
     SW2 = response[-2:]
-    logging.info(F"Tx->{apdu}")
-    logging.info(F"Rx<-{data}{SW1}{SW2}")
+    logging.debug(F"Tx->{apdu}")
+    logging.debug(F"Rx<-{data}{SW1}{SW2}")
 
     return data, SW1, SW2
 
@@ -223,7 +231,7 @@ def send_apdu(hcard, protocol: Protocol, apdu: dict):
     Le = apdu.get('Le', None)
 
     if protocol not in Protocol:
-        raise Exception(F'Unsupported protocol: {protocol}')
+        raise PcscError(F'Unsupported protocol: {protocol}')
 
     if data is None:
         # Case 1
@@ -245,7 +253,7 @@ def send_apdu(hcard, protocol: Protocol, apdu: dict):
         logging.debug("Case 2E")
         if len(Le) == 6:
             if protocol == Protocol.T0:
-                raise Exception('Unsupported case 2E with protocol T=0')
+                raise PcscError('Unsupported case 2E with protocol T=0')
             if protocol == Protocol.T1:
                 return transmit(hcard, protocol, header + Le)
 
@@ -265,12 +273,12 @@ def send_apdu(hcard, protocol: Protocol, apdu: dict):
         if Lc < 65536:
             Lc = F"00{Lc:04X}"
             if protocol == Protocol.T0:
-                raise Exception('Unsupported case 3E with protocol T=0')
+                raise PcscError('Unsupported case 3E with protocol T=0')
             if protocol == Protocol.T1:
                 return transmit(hcard, protocol, header + Lc + data)
         # Data field too long
         else:
-            raise Exception(F"Data field too long: {Lc:X}")
+            raise PcscError(F"Data field too long: {Lc:X}")
 
     # Case 4S
     logging.debug("Case 4S")
@@ -283,11 +291,11 @@ def send_apdu(hcard, protocol: Protocol, apdu: dict):
     logging.debug("Case 4E")
     if Lc < 65536 and len(Le) == 6:
         if protocol == Protocol.T0:
-            raise Exception('Unsupported case 4E with protocol T=0')
+            raise PcscError('Unsupported case 4E with protocol T=0')
         if protocol == Protocol.T1:
             return transmit(hcard, protocol, header + F"00{Lc:04X}" + data + Le)
 
-    raise Exception(
+    raise PcscError(
         F'Unsupported case with short and extended lengths: Lc = {Lc:X}, Le = {Le}')
 
 
@@ -305,7 +313,7 @@ def _send_apdu_T0_case_2s(hcard, header: str, Le: str):
         pass
     # Wrong Le field
     else:
-        raise Exception(F'Wrong Le value: {Le}')
+        raise PcscError(F'Wrong Le value: {Le}')
 
     data, SW1, SW2 = transmit(hcard, Protocol.T0, header + Le)
     # Case 2S.1—Process completed: Ne accepted
@@ -313,7 +321,7 @@ def _send_apdu_T0_case_2s(hcard, header: str, Le: str):
         return data, SW1, SW2
     # Case 2S.2—Process aborted: Ne definitively not accepted
     if SW1+SW2 == '6700':
-        raise Exception(
+        raise PcscError(
             'Error—Process aborted: Ne definitively not accepted (6700)')
     # Case 2S.3—Process aborted; Ne not accepted, Na indicated
     if SW1 == '6C':
@@ -322,7 +330,7 @@ def _send_apdu_T0_case_2s(hcard, header: str, Le: str):
     if SW1.startswith('9'):
         return data, SW1, SW2
 
-    raise Exception(F'Unknown command case (data={data}, SW12={SW1}{SW2})')
+    raise PcscError(F'Unknown command case (data={data}, SW12={SW1}{SW2})')
 
 
 def _send_apdu_T0_case_3s(hcard, header: str, Lc: str, data: str):
@@ -333,7 +341,7 @@ def _send_apdu_T0_case_3s(hcard, header: str, Lc: str, data: str):
         pass
     # Data field too long
     else:
-        raise Exception(F"Data field too long: {Lc}")
+        raise PcscError(F"Data field too long: {Lc}")
 
     return transmit(hcard, Protocol.T0, header + Lc + data)
 
@@ -346,19 +354,19 @@ def _send_apdu_T0_case_4s(hcard, header: str, Lc: str, data: str, Le: str):
         pass
     # Data field too long
     else:
-        raise Exception(F"Data field too long: {Lc}")
+        raise PcscError(F"Data field too long: {Lc}")
 
     # Short Le field
     if len(Le) == 2:
         pass
     # Wrong Le field
     else:
-        raise Exception(F'Wrong Le value: {Le}')
+        raise PcscError(F'Wrong Le value: {Le}')
 
     data, SW1, SW2 = transmit(hcard, Protocol.T0, header + Lc + data + Le)
     # Case 4S.1—Process aborted
     if SW1[0] == '6' and SW1[1] in '0456789ABCDEF':
-        raise Exception(
+        raise PcscError(
             F'Error—Process aborted (data={data}, SW12={SW1}{SW2})')
     # Case 4S.2—Process completed
     if SW1+SW2 == '9000':
@@ -375,7 +383,7 @@ def _send_apdu_T0_case_4s(hcard, header: str, Lc: str, data: str, Le: str):
     if SW1.startswith('9') or SW1 in ['62', '63']:
         return data, SW1, SW2
 
-    raise Exception(F'Unknown command case (data={data}, SW12={SW1}{SW2})')
+    raise PcscError(F'Unknown command case (data={data}, SW12={SW1}{SW2})')
 
 
 def get_status_change(hcontext, reader_states=None, timeout=None):
@@ -396,13 +404,13 @@ def get_status_change(hcontext, reader_states=None, timeout=None):
         hcontext, timeout, old_reader_states)
     if hresult != _SCARD_S_SUCCESS:
         err = F'Failed to get new reader states ({_SCardGetErrorMessage(hresult)})'
-        logging.info(err)
-        raise Exception(err)
+        logging.debug(err)
+        raise PcscError(err)
 
     new_reader_states = list(
         map(_convert_reader_state_from_scard, new_reader_states))
 
-    logging.info(F"new reader states: {', '.join(new_reader_states)}")
+    logging.debug(F"new reader states: {', '.join(new_reader_states)}")
     return new_reader_states
 
 
@@ -414,10 +422,10 @@ def disconnect(hcard, disposition: Disposition):
     hresult = _SCardDisconnect(hcard, dw_disposition)
     if hresult != _SCARD_S_SUCCESS:
         err = F'Failed to disconnect ({_SCardGetErrorMessage(hresult)})'
-        logging.info(err)
-        raise Exception(err)
+        logging.debug(err)
+        raise PcscError(err)
 
-    logging.info("Disconnected")
+    logging.debug("Disconnected")
 
 
 def release_context(hcontext):
@@ -426,10 +434,10 @@ def release_context(hcontext):
     hresult = _SCardReleaseContext(hcontext)
     if hresult != _SCARD_S_SUCCESS:
         err = F'Failed to release context: {_SCardGetErrorMessage(hresult)}'
-        logging.info(err)
-        raise Exception(err)
+        logging.debug(err)
+        raise PcscError(err)
 
-    logging.info("Context released")
+    logging.debug("Context released")
 
 #
 # Helper functions
